package org.artifactory.repo.service.mover;

import org.artifactory.api.common.MoveMultiStatusHolder;
import org.artifactory.exception.CancelException;
import org.artifactory.repo.LocalRepo;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoRepoPath;
import org.artifactory.storage.db.fs.model.DbMutableFile;
import org.artifactory.storage.db.fs.session.SqlStorageSession;
import org.artifactory.storage.fs.MutableVfsItem;
import org.artifactory.storage.fs.lock.SessionLockEntry;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.artifactory.repo.service.flexible.tools.MoveCopyTestUtils.prepareCopyOrMoveOverrideMocks;
import static org.artifactory.repo.service.flexible.tools.MoveCopyTestUtils.prepareDbMutableFileMock;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class MoverProxyTest extends ArtifactoryHomeBoundTest {

    @Mock
    RepoPath repoPathMock;
    @Mock
    LocalRepo repoMock;
    @Mock
    MutableVfsItem mutableVfsItemMock;
    @Mock
    MoveMultiStatusHolder moveMultiStatusHolderMock;
    @Mock
    SqlStorageSession sqlStorageSessionMock;
    @Mock
    SessionLockEntry sessionLockEntryMock;

    @BeforeMethod
    public void setup() {
        MockitoAnnotations.initMocks(this);
    }

    @Test(dataProvider = "moveCopyFileProvider")
    public void testMoveCopyFile(boolean beforeOperationError, int invokeNumGetMutableFsItem,
            int invokeNumOperationOnFile, int invokeNumSaveSession) {

        BaseRepoPathMover baseRepoPathMover = mock(BaseRepoPathMover.class);
        DbMutableFile dbMutableFile = prepareDbMutableFileMock();
        RepoRepoPath<LocalRepo> repoRepoRepoPath =
                prepareCopyOrMoveOverrideMocks(repoPathMock, repoMock, mutableVfsItemMock, sessionLockEntryMock,
                        sqlStorageSessionMock, moveMultiStatusHolderMock, true);
        MoverProxy moverProxy = new MoverProxy();
        if (beforeOperationError) {
            when(moveMultiStatusHolderMock.getCancelException(null)).thenReturn(mock(CancelException.class));
        }

        moverProxy.moveCopyFile(dbMutableFile, repoRepoRepoPath, baseRepoPathMover, moveMultiStatusHolderMock);

        verify(repoMock, times(invokeNumGetMutableFsItem)).getMutableFsItem(any());
        verify(baseRepoPathMover, times(invokeNumOperationOnFile)).operationOnFile(any(), any(), any());
        verify(baseRepoPathMover, times(invokeNumSaveSession)).saveSession();
    }

    @DataProvider
    public Object[][] moveCopyFileProvider() {
        return new Object[][]{
                {false, 1, 1, 1},
                {true, 0, 0, 0}
        };
    }
}