/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.test;

import org.artifactory.common.ConstantValues;
import org.artifactory.common.ArtifactoryHome;
import org.artifactory.mime.MimeTypes;
import org.artifactory.mime.MimeTypesReader;
import org.artifactory.version.ArtifactoryVersion;
import org.jfrog.common.ResourceUtils;
import org.jfrog.config.watch.FileWatchingManager;
import org.jfrog.config.wrappers.ConfigurationManagerImpl;
import org.jfrog.config.ConfigurationManager;
import org.jfrog.config.wrappers.ConfigurationManagerAdapter;
import org.mockito.Mockito;
import org.testng.annotations.*;

import java.io.InputStream;

import static org.artifactory.common.ArtifactoryHome.MIME_TYPES_FILE_NAME;

/**
 * A convenience class for tests that require bind/unbind of {@link ArtifactoryHome}.
 * Note: this is the JUnit 5 version. For the TestNG version see {@link ArtifactoryHomeBoundTest}.
 *
 * @author Yossi Shaul
 */
public class ArtifactoryHomeBoundJUnitTest {
    static MimeTypes mimeTypes;
    static ArtifactoryHomeStub homeStub;
    static ConfigurationManager configurationManager;

    @BeforeClass
    public void readMimeTypes() {
        if (mimeTypes != null) {
            InputStream mimeTypesFile = ResourceUtils.getResource("/META-INF/default/" + MIME_TYPES_FILE_NAME);
            mimeTypes = new MimeTypesReader().read(mimeTypesFile);
        }
    }

    @BeforeMethod
    public void bindArtifactoryHome() {
        ArtifactoryHomeStub home = getOrCreateArtifactoryHomeStub();
        if (configurationManager == null) {
            ConfigurationManagerAdapter adapter = new MockConfigurationManagerAdapter(home);
            FileWatchingManager fileWatchingManager = Mockito.mock(FileWatchingManager.class);
            configurationManager = ConfigurationManagerImpl.create(adapter, fileWatchingManager);
        }
        configurationManager.initDefaultFiles();
        home.initArtifactorySystemProperties();
        home.initMimeTypes();
        ArtifactoryHome.bind(home);
    }

    public ArtifactoryHomeStub getOrCreateArtifactoryHomeStub() {
        if (homeStub == null) {
            String homeIdentifier = getClass().getCanonicalName();
            homeStub = new ArtifactoryHomeStub(homeIdentifier);
            homeStub.setMimeTypes(mimeTypes);
            loadAndBindArtifactoryProperties(homeStub);
        }
        return homeStub;
    }

    private void loadAndBindArtifactoryProperties(ArtifactoryHomeStub artifactory) {
        artifactory.loadSystemProperties();
        artifactory.setProperty(ConstantValues.artifactoryVersion, ArtifactoryVersion.getCurrent().getVersion());
    }

    @AfterMethod
    public void unbindArtifactoryHome() {
        ArtifactoryHome.unbind();
    }

    @AfterClass
    static void cleanEnv() {
        if (configurationManager != null) {
            configurationManager.destroy();
        }
    }
}